###########################################################################
## (c) Maria Dascalu 2016
## Generates one graph depending on type.
## populate() : generates randomly
## populatehalf() : generates half red edges and half blue edges
## generate51() : generates a graph for k=5 based on critical graphs for k=4.
###########################################################################

import random

def populate(n, c):
    """(int, int)-> dictionary
    Populates a dictionary with n keys and c colors, randomly. Basically an
    adjacency list for the graph.
    """
    graph = {}
    for i in range(1, n+1):
        graph[i] = {}
        for j in range(1, n+1):
            if i < j:
                graph[i][j] = random.randint(0, c-1)
            elif i > j:
                graph[i][j] = graph[j][i]
    return graph

def populatehalf(n):
    """(int, int) -> dictionary
    Populates a dicionary with n keys, not randomly. Instead it tries to
    populate a graph so that it has roughly equal numbers of 2-colored
    edges.
    """

    numedges = n*(n-1) // 2
    m = numedges // 2
    if numedges % 2 != 0:
        m += 1
    ## m is the maximum number of color we can have.
    ## In a graph with an even n, we will have a 50-50 graph. In a graph with odd
    ## n, one color will be "heavier" than the other. But that's okay.

    first = random.randint(0, 1) # first color, random for randomness' sake
    second = (first + 1)%2
    countfirst = 0
    countsecond = 0

    graph = {}
    for i in range(1, n+1):
        graph[i] = {}
        for j in range(1, n+1):
            if i < j:
                if countfirst < m:
                    if countsecond < m:
                        color = random.randint(0,1)
                        if color == first:
                            countfirst += 1
                        else:
                            countsecond += 1
                    else:
                        color = first
                        countfirst += 1
                else:
                    color = second
                    countsecond += 1
                graph[i][j] = color
            if i > j:
                graph[i][j] = graph[j][i]
    return graph

def generate17():
    """() -> dictionary
    Generates a critical graph for 17 vertices. according to the test
    described in the book Ramsey Theory by Graham, Rothschild, and
    Spencer.
    """
    solution = [1, 2, 4, 8, 9, 13, 15, 16]
    n = 17
    graph = {}

    for i in range(1, n+1):
        graph[i] = {}
        for j in range(1, n+1):
            if i < j:
                if abs(i-j) in solution:
                    ## Let 0 be red and 1 be blue. Then
                    graph[i][j] = 0
                else:
                    graph[i][j] = 1
            if i > j:
                graph[i][j] = graph[j][i]
    #print(graph)
    return graph

def generate51():
    """() -> dictionary
    Using a critical graph, it connects three copies of that graph with
    randomly colored edges. This graph has 51 edges, which is obviously
    too many for a critical graph for k=5, but we will delete vertices
    later.
    """
    graph17 = generate17()

    # Generating a completely new 51-vertex graph where individual
    graph = {}
    for i in range(1, 52):
        graph[i] = {}
        groupi = (i-1) // 17
        #print("GROUPi!!! {} : {}".format(i, groupi))
        graph[i] = {}
        for j in range(1, 52):
            groupj = (j-1) // 17
            #print("GROUPj {}: {}".format(j, groupj))
            if groupi != groupj:
                if i < j:
                    graph[i][j] = random.randint(0,1)
                elif i > j:
                    graph[i][j] = graph[j][i]
            else:
                #print("i {} j {} i%17 {} j%17 {}".format(i, j, i%17, j%17))
                if i != j:
                    if i%17 == 0:
                        im = 17
                    else:
                        im = i%17
                    if j%17 == 0:
                        jm = 17
                    else:
                        jm = j%17
                    graph[i][j] = graph17[im][jm]
    return graph

def choose8():
    """() -> list
    Chooses eight distinct random integers between 1 and 51, which are
    the integers we will ignore later on.
    """
    chosen = []

    for i in range(8):
        while True:
            choose = random.randint(1,51)
            if choose not in chosen:
                chosen.append(choose)
                break
    #print(chosen)
    return chosen


###########################################################################
##############                 GRAPH CREATION                ##############
###########################################################################

def vbycolor(graph):
    """(dictionary) -> dictionary
    Restructures the adjacency list so that the vertices of the graph are
    grouped by color.
    """
    graph_c = {}
    for i in graph:
        graph_c[i] = {}
        for j in graph[i]:
            color = graph[i][j]
            if color in graph_c[i]:
                graph_c[i][color].append(j)
            else:
                graph_c[i][color] = [j]

    return graph_c

#choose8()
